#ifndef __MATRIX_H__
#define __MATRIX_H__

#include <math.h>
#include <ostream>

template<class T> struct Vector2;
template<class T> struct Vector3;
template<class T> struct Quaternion;
template<class T> struct Matrix33;

template <class T>void Swap(T &_a,T &_b){T tmp=_a;_a=_b;_b=tmp;}

template <class T>
struct Vector2
{
public:
	T x,y;
public:
	Vector2(){}
	Vector2(T _x,T _y):x(_x),y(_y){}
	Vector2(const Vector2 &_v):x(_v.x),y(_v.y){}
	Vector2(const T *_a):x(_a[0]),y(_a[1]){}

	Vector2 &operator=(const Vector2 &_v){x=_v.x;y=_v.y;return *this;}

	void ToArray(T *_a){_a[0]=x;_a[1]=y;}

	Vector2 &Translate(T _x,T _y){x+=_x;y+=_y;return *this;}
	Vector2 &Translate(const Vector2 &_v){x+=_v.x;y+=_v.y;return *this;}

	Vector2 &operator+=(const Vector2 &_v){x+=_v.x;y+=_v.y;return *this;}
	Vector2 &operator-=(const Vector2 &_v){x-=_v.x;y-=_v.y;return *this;}
	Vector2 &operator*=(T _m){x*=_m;y*=_m;return *this;}
	Vector2 &operator*=(const Vector2 &_v){Vector2 v(x*_v.x-y*_v.y,x*_v.y+y*_v.x);*this=v;return *this;}

	Vector2 operator+(const Vector2 &_v)const{Vector2 v(x+_v.x,y+_v.y);return v;}
	Vector2 operator-(const Vector2 &_v)const{Vector2 v(x-_v.x,y-_v.y);return v;}
	Vector2 operator-()const{Vector2 v(-x,-y);return v;}
	Vector2 operator*(T _m)const{Vector2 v(x*_m,y*_m);return v;}
	Vector2 operator*(const Vector2 &_v){Vector2 v(x*_v.x-y*_v.y,x*_v.y+y*_v.x);return v;}

	bool operator==(const Vector2 &_v)const{return (x==_v.x)&&(y==_v.y);}
	bool operator!=(const Vector2 &_v)const{return (x!=_v.x)||(y!=_v.y);}

	T Abs2()const{return x*x+y*y;}
	double Abs()const{return sqrt(x*x+y*y);}
	T Dist2(const Vector2 &_v)const{Vector2 v=*this-_v;return v.Abs2();}
	double Dist(const Vector2 &_v)const{Vector2 v=*this-_v;return v.Abs();}

	Vector2 Norm()const{double d=sqrt(x*x+y*y);Vector2 v(x/d,y/d);return v;}
	Vector2 &Normalize(){double d=sqrt(x*x+y*y);x/=d;y/=d;return *this;}

	Vector2 &Rotate(double _a){double s=sin(_a),c=cos(_a);Vector2 r(x*c-y*s,x*s+y*c);*this=r;return *this;}
	T DotProd(const Vector2 &_v)const{return x*_v.x+y*_v.y;}
	T ExProd(const Vector2 &_v)const{return x*_v.y-y*_v.x;}
};

template <class T>
struct Vector3
{
public:
	T x,y,z;
public:
	Vector3(){}
	Vector3(T _x,T _y,T _z):x(_x),y(_y),z(_z){}
	Vector3(const Vector3 &_v):x(_v.x),y(_v.y),z(_v.z){}
	Vector3(const T *_a):x(_a[0]),y(_a[1]),z(_a[2]){}

	Vector3 &operator=(const Vector3 &_v){x=_v.x;y=_v.y;z=_v.z;return *this;}

	void ToArray(T *_a){_a[0]=x;_a[1]=y;_a[2]=z;}

	Vector3 &Translate(T _x,T _y,T _z){x+=_x;y+=_y;z+=_z;return *this;}
	Vector3 &Translate(const Vector3 &_v){x+=_v.x;y+=_v.y;z+=_v.z;return *this;}

	Vector3 &operator+=(const Vector3 &_v){x+=_v.x;y+=_v.y;z+=_v.z;return *this;}
	Vector3 &operator-=(const Vector3 &_v){x-=_v.x;y-=_v.y;z-=_v.z;return *this;}
	Vector3 &operator*=(T _m){x*=_m;y*=_m;z*=_m;return *this;}

	Vector3 operator+(const Vector3 &_v)const{Vector3 v(x+_v.x,y+_v.y,z+_v.z);return v;}
	Vector3 operator-(const Vector3 &_v)const{Vector3 v(x-_v.x,y-_v.y,z-_v.z);return v;}
	Vector3 operator-()const{Vector3 v(-x,-y,-z);return v;}
	Vector3 operator*(T _m)const{Vector3 v(x*_m,y*_m,z*_m);return v;}

	bool operator==(const Vector3 &_v)const{return (x==_v.x)&&(y==_v.y)&&(z==_v.z);}
	bool operator!=(const Vector3 &_v)const{return (x!=_v.x)||(y!=_v.y)||(z!=_v.z);}

	T Abs2()const{return x*x+y*y+z*z;}
	double Abs()const{return sqrt(x*x+y*y+z*z);}
	T Dist2(const Vector3 &_v)const{Vector3 v=*this-_v;return v.Abs2();}
	double Dist(const Vector3 &_v)const{Vector3 v=*this-_v;return v.Abs();}

	Vector3 Norm()const{double d=sqrt(x*x+y*y+z*z);if(d==0)return Vector3(1,0,0);Vector3 v(x/d,y/d,z/d);return v;}
	Vector3 &Normalize(){double d=sqrt(x*x+y*y+z*z);if(d==0){x=1;y=0;z=0;return *this;}x/=d;y/=d;z/=d;return *this;}

	T DotProd(const Vector3 &_v)const{return x*_v.x+y*_v.y+z*_v.z;}
	Vector3 ExProd(const Vector3 &_v)const{Vector3 v(y*_v.z-z*_v.y,z*_v.x-x*_v.z,x*_v.y-y*_v.x);return v;}

	void RotateZ(double r){double s=sin(r),c=cos(r);double _x=c*x-s*y,_y=s*x+c*y;x=_x;y=_y;}
	void RotateX(double r){double s=sin(r),c=cos(r);double _y=c*y-s*z,_z=s*y+c*z;y=_y;z=_z;}
	void RotateY(double r){double s=sin(r),c=cos(r);double _z=c*z-s*x,_x=s*z+c*x;z=_z;x=_x;}

};

template<class T>
struct Quaternion
{
public:
	T x,y,z,w;
public:
	Quaternion(){}
	Quaternion(T _x,T _y,T _z,T _w):w(_w),x(_x),y(_y),z(_z){}
	Quaternion(const Vector3<T> &_v,T _w):w(_w),x(_v.x),y(_v.y),z(_v.z){}
	Quaternion(const Quaternion &_q):w(_q.w),x(_q.x),y(_q.y),z(_q.z){}
	Quaternion(const T *_a):w(_a[3]),x(_a[0]),y(_a[1]),z(_a[2]){}

	Quaternion &operator=(const Quaternion &_q){w=_q.w;x=_q.x;y=_q.y;z=_q.z;return *this;}

	void ToArray(T *_a){_a[0]=x;_a[1]=y;_a[2]=z;_a[3]=w;}

	Quaternion &operator*=(const Quaternion &_q)
	{
		Vector3<double> v0(x,y,z),v1(_q.x,_q.y,_q.z),v;
		v=v0.ExProd(v1)+v1*w+v0*_q.w;x=v.x;y=v.y;z=v.z;
		w=w*_q.w-v0.DotProd(v1);
		return *this;
	}
	Quaternion operator*(const Quaternion &_q)const
	{
		Quaternion q;
		Vector3<double> v0(x,y,z),v1(_q.x,_q.y,_q.z),v;
		v=v0.ExProd(v1)+v1*w+v0*_q.w;q.x=v.x;q.y=v.y;q.z=v.z;
		q.w=w*_q.w-v0.DotProd(v1);
		return q;
	}

	Quaternion &operator+=(const Quaternion &_q){w+=_q.w;x+=_q.x;y+=_q.y;z+=_q.z;return *this;}
	Quaternion operator+(const Quaternion &_q)const{return Quaternion(x+_q.x,y+_q.y,z+_q.z,w+_q.w);}
	Quaternion &operator-=(const Quaternion &_q){w-=_q.w;x-=_q.x;y-=_q.y;z-=_q.z;return *this;}
	Quaternion operator-(const Quaternion &_q)const{return Quaternion(x-_q.x,y-_q.y,z-_q.z,w-_q.w);}
	Quaternion &operator*=(T _m){w*=_m;x-=_m;y-=_m;z-=_m;return *this;}
	Quaternion operator*(T _m)const{return Quaternion(x*_m,y*_m,z*_m,w*_m);}

	Quaternion &Identity(){w=1;x=0;y=0;z=0;return *this;}

	//_v_thWA]NH[^jI
	Quaternion &RotationAxis(const Vector3<T> &_v,double _th)
	{
		Vector3d v=_v.Norm()*sin(_th/2);
		w=cos(_th/2);x=_v.x;y=_v.y;z=_v.z;
		return *this;
	}

	Vector3<T> Rotate(const Vector3<T> &_v)const
	{
		Quaternion cj=Conj();
		Quaternion v(_v,0);
		Quaternion r=(*this)*v*cj;
		return Vector3<T>(r.x,r.y,r.z);
	}

	//
	Quaternion Conj()const{return Quaternion(-x,-y,-z,w);}
	Quaternion &Conjugate(){x=-x;y=-y;z=-z;return *this;}

	//
	T Length2()const{return w*w+x*x+y*y+z*z;}
	double Length()const{return sqrt(w*w+x*x+y*y+z*z);}

	//t
	Quaternion Inv()const{double l=Length2();return Quaternion(-x/l,-y/l,-z/l,w/l);}
	Quaternion &Invert(){double l=Length2();w=w/l;x=-x/l;y=-y/l;z=-z/l;return *this;}

	//K
	Quaternion Norm()const{double l=Length2();return Quaternion(x/l,y/l,z/l,w/l);}
	Quaternion &Normalize(){double l=Length2();w=w/l;x=x/l;y=y/l;z=z/l;return *this;}

	T DotProd(const Quaternion &_q)const{return w*_q.w+x*_q.x+y*_q.y+z*_q.z;}

	//`
	static Quaternion Lerp(const Quaternion &_a,const Quaternion &_b,double _t)
	{
		return _a+(_b-_a)*_t;
	}
	Quaternion Lerp(const Quaternion &_q,double _t)const
	{
		return *this+(_q-*this)*_t;
	}
	//ʐ`
	static Quaternion Slerp(const Quaternion &_a,const Quaternion &_b,double _t)
	{
		Quaternion tmpq;
		T dot=_a.DotProd(_b);
		if(dot<0){dot=-dot;tmpq=_b*-1;}
		else{tmpq=_b;}
		if(dot<0.99)
		{
		double th=acos(dot);
		return (  _a*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_a,tmpq,_t);
		}
	}
	Quaternion Slerp(const Quaternion &_q,double _t)const
	{
		Quaternion tmpq;
		T dot=DotProd(_q);
		if(dot<0){dot=-dot;tmpq=_q*-1;}
		else{tmpq=_q;}
		if(dot<0.99)
		{
			double th=acos(dot);
			return (  (*this)*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(tmpq,_t);
		}
	}
	//]Ȃʐ`
	static Quaternion SlerpNoInv(const Quaternion &_a,const Quaternion &_b,double _t)
	{
		Quaternion tmpq;
		T dot=_a.DotProd(_b);
		if(dot>-0.99&&dot<0.99)
		{
		double th=acos(dot);
		return (  _a*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_a,_b,_t);
		}
	}
	Quaternion SlerpNoInv(const Quaternion &_q,double _t)const
	{
		Quaternion tmpq;
		T dot=DotProd(_q);
		if(dot>-0.99&&dot<0.99)
		{
			double th=acos(dot);
			return (  (*this)*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_q,_t);
		}
	}
	//3
	static Quaternion Squad(const Quaternion &_a,const Quaternion &_b,const Quaternion &_c,const Quaternion &_d,double _t)
	{
		return SlerpNoInv(SlerpNoInv(_a,_d,t),SlerpNoInv(_b,_c,t),2*t*(1-t));
	}
	Quaternion Squad(const Quaternion &_b,const Quaternion &_c,const Quaternion &_d,double _t)const
	{
		return SlerpNoInv(*this,_d,t).SlerpNoInv(SlerpNoInv(_b,_c,t),2*t*(1-t));
	}

	//w(NH[^jI w=0 Ɍ)
	Quaternion Exp()const
	{
		double th=Length(),c=cos(th);
		if(th>0)
		{
			double sc=sin(th)/th;
			return Quaternion(x*sc,y*sc,z*sc,c);
		}
		else
		{
			return Quaternion(0,0,0,c);
		}
	}
	static Quaternion Exp(const Quaternion &_q)
	{
		double th=_q.Length(),c=cos(th);
		if(th>0){double sc=sin(th)/th;return Quaternion(_q.x*sc,_q.y*sc,_q.z*sc,c);}
		else{return Quaternion(0,0,0,c);}
	}

	//ΐ(PʃNH[^jI l=0 Ɍ)
	Quaternion Ln()const
	{
		double th=acos(w),s=sin(th);
		if(s>0)
		{
			double sc=th/s;
			return Quaternion(x*sc,y*sc,z*sc,0);
		}
		else
			return Quaternion(0,0,0,0);
	}
	static Quaternion Ln(const Quaternion &_q)
	{
		double th=acos(_q.w),s=sin(th);
		if(s>0){double sc=th/s;return Quaternion(_q.x*sc,_q.y*sc,_q.z*sc,0);}
		else{return Quaternion(0,0,0,0);}
	}
	//XvCԗp
	Quaternion Spline(const Quaternion &_m,const Quaternion &_0,const Quaternion &_p)
	{
		Quaternion inv=_0.Inv();
		return _0*Exp(  (Ln(inv*_m)+Ln(inv*_p)) * -0.25  );
	}

	Matrix33<T> ToRotMatrix33()const
	{
		return Matrix33<T>(
			1-2*y*y-2*z*z,	2*x*y+2*w*z,	2*x*z-2*w*y,
			2*x*y-2*w*z,	1-2*x*x-2*z*z,	2*y*z+2*w*x,
			2*x*z+2*w*y,	2*y*z-2*w*x,	1-2*x*x-2*y*y
			);
	}

};

template<class T>
struct Matrix33
{
public:
	T m[3][3];
public:
	Matrix33(){}
	Matrix33(T _00,T _01,T _02,T _10,T _11,T _12,T _20,T _21,T _22)
	{
		m[0][0]=_00;m[0][1]=_01;m[0][2]=_02;
		m[1][0]=_10;m[1][1]=_11;m[1][2]=_12;
		m[2][0]=_20;m[2][1]=_21;m[2][2]=_22;
	}
	Matrix33(const Vector3<T> &_v0,const Vector3<T> &_v1,const Vector3<T> &_v2)
	{
		m[0][0]=_v0.x;m[0][1]=_v0.y;m[0][2]=_v0.z;
		m[1][0]=_v1.x;m[1][1]=_v1.y;m[1][2]=_v1.z;
		m[2][0]=_v2.x;m[2][1]=_v2.y;m[2][2]=_v2.z;
	}
	Matrix33(const Matrix33 &_M)
	{
		m[0][0]=_M.m[0][0];m[0][1]=_M.m[0][1];m[0][2]=_M.m[0][2];
		m[1][0]=_M.m[1][0];m[1][1]=_M.m[1][1];m[1][2]=_M.m[1][2];
		m[2][0]=_M.m[2][0];m[2][1]=_M.m[2][1];m[2][2]=_M.m[2][2];
	}
	
	Matrix33 &Identity()
	{
		m[0][0]=0;m[0][1]=0;m[0][2]=0;
		m[1][0]=0;m[1][1]=0;m[1][2]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=0;
		return *this;
	}
	void ToArray44(T *_a)
	{
		int n=0;
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++,n++)
			{
				_a[n]=m[i][j];
			}
			_a[n]=0;n++;
		}
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=1;n++;
	}
	
	//]u
	Matrix33 Transp()const
	{
		Matrix33 M(*this);
		Swap(M.m[1][0],M.m[0][1]);
		Swap(M.m[1][2],M.m[2][1]);
		Swap(M.m[2][0],M.m[0][2]);
		return M;
	}
	Matrix33 &Transpose()
	{
		Swap<T>(m[1][0],m[0][1]);
		Swap<T>(m[1][2],m[2][1]);
		Swap<T>(m[2][0],m[0][2]);
		return *this;
	}
	
	//g[X
	T Trace()const
	{
		return m[0][0]+m[1][1]+m[2][2];
	}
	
	//s
	T Determinant()const
	{
		return
			+m[0][0]*m[1][1]*m[2][2]
			+m[0][1]*m[1][2]*m[2][0]
			+m[0][2]*m[1][0]*m[2][1]
			-m[0][2]*m[1][1]*m[2][0]
			-m[0][1]*m[1][0]*m[2][2]
			-m[0][0]*m[1][2]*m[2][1];
	}
	
	//NH[^jIɕϊB(]sɌ)
	Quaternion<T> ToQuaternion()const
	{
		T tr=Trace();
		if(tr>0)
		{
			Quaternion<T> q;
			q.w=sqrt(tr+1)/2;
			q.x=(m[1][2]-m[2][1])/(4*q.w);
			q.y=(m[2][0]-m[0][2])/(4*q.w);
			q.z=(m[0][1]-m[1][0])/(4*q.w);
			return q;
		}
		if(m[0][0]>=m[1][1]&&m[0][0]>=m[2][2])
		{
			Quaternion<T> q;
			q.x=sqrt(m[0][0]-m[1][1]-m[2][2]+1)/2;
			q.w=(m[1][2]-m[2][1])/(4*q.x);
			q.y=(m[0][1]+m[1][0])/(4*q.x);
			q.z=(m[0][2]+m[2][0])/(4*q.x);
			return q;
		}
		if(m[1][1]>=m[0][0]&&m[1][1]>=m[2][2])
		{
			Quaternion<T> q;
			q.y=sqrt(m[1][1]-m[0][0]-m[2][2]+1)/2;
			q.w=(m[2][0]-m[0][2])/(4*q.y);
			q.x=(m[0][1]+m[1][0])/(4*q.y);
			q.z=(m[1][2]+m[2][1])/(4*q.y);
			return q;
		}
		{
			Quaternion<T> q;
			q.z=sqrt(m[2][2]-m[0][0]-m[1][1]+1)/2;
			q.w=(m[0][1]-m[1][0])/(4*q.z);
			q.x=(m[2][0]+m[0][2])/(4*q.z);
			q.y=(m[2][1]+m[1][2])/(4*q.z);
			return q;
		}
	}
};


template<class T> struct Rect2
{
public:
	Vector2<T> p1,p2;
public:
	Rect2<T>(const Vector2<T> &_1,const Vector2<T> &_2):p1(_1),p2(_2){}
	Rect2<T>(const Rect2<T> &_r):p1(_r.p1),p2(_r.p2){}
	Rect2<T>(T x1,T y1,T x2,T y2):p1(x1,y1),p2(x2,y2){}
	Rect2<T>(){}
	//_v̈ɓĂtrueAĂȂfalse
	bool In(const Vector2<T> &_v)const
	{
		if(p1.x<=_v.x&&_v.x<=p2.x&&p1.y<=_v.y&&_v.y<=p2.y)
			return true;
		return false;
	}
	//Ίp̃xNg
	Vector2<T> Diagonal()const{return p2-p1;}
	//̗̈0-1Ɖ肵ƂfŁÄ̗𕪂
	Rect2<T> Subdiv(Rect2<T> &_r)const
	{
		Vector2f dthis=Diagonal();
		//Vector2f dsub=_r.Diagonal();

		return Rect2<T>(
			Vector2<T>(p1.x+dthis.x*_r.p1.x,p1.y+dthis.y*_r.p1.y),
			Vector2<T>(p1.x+dthis.x*_r.p2.x,p1.y+dthis.y*_r.p2.y));
	}
	//_v:0-1̗̈ɕϊ
	Vector2<T> TransformTo(Vector2<T> &_v)const
	{
		Vector2<T> dv=_v-p1,dr=Diagonal();
		return Vector2f(dv.x/dr.x,dv.y/dr.y);
	}
};


typedef Vector2<float> Vector2f;
typedef Vector3<float> Vector3f;
typedef Quaternion<float> Quaternionf;
typedef Matrix33<float> Matrix33f;
typedef Rect2<float> Rect2f;

typedef Vector2<int> Vector2i;
typedef Vector3<int> Vector3i;

#endif

